@@ -32,8 +32,6 @@
 
 #include <glib/gstdio.h>
 
-#include "mini-fribidi/fribidi.h"
-
 #ifndef HAVE_FLOCKFILE
 #  define flockfile(f) (void)1
 #  define funlockfile(f) (void)1
@@ -59,8 +57,6 @@ struct PangoAlias
 
 static GHashTable *pango_aliases_ht = NULL;
 
-PangoWarningHistory _pango_warning_history = { FALSE, FALSE, FALSE };
-
 /**
  * pango_version:
  *
@@ -71,7 +67,7 @@ PangoWarningHistory _pango_warning_history = { FALSE, FALSE, FALSE };
  * A version number can be encoded into an integer using
  * PANGO_VERSION_ENCODE().
  *
- * Returns value: The encoded version of Pango library
+ * Returns: The encoded version of Pango library
  *   available at run time.
  *
  * Since: 1.16
@@ -89,7 +85,7 @@ pango_version (void)
  * it returns the version of Pango available at run-time, as opposed to
  * the version available at compile-time.
  *
- * Returns value: A string containing the version of Pango library
+ * Returns: A string containing the version of Pango library
  *   available at run time.
  *   The returned string is owned by Pango and should not be modified
  *   or freed.
@@ -512,7 +508,6 @@ pango_scan_string (const char **pos, GString *out)
 gboolean
 pango_scan_int (const char **pos, int *out)
 {
-  unsigned int i = 0;
   char *end;
   long temp;
 
@@ -1090,126 +1085,6 @@ pango_parse_stretch (const char   *str,
   return FALSE;
 }
 
-/**
- * pango_log2vis_get_embedding_levels:
- * @text:      the text to itemize.
- * @length:    the number of bytes (not characters) to process, or -1
- *             if @text is nul-terminated and the length should be calculated.
- * @pbase_dir: input base direction, and output resolved direction.
- *
- * This will return the bidirectional embedding levels of the input paragraph
- * as defined by the Unicode Bidirectional Algorithm available at:
- *
- *   http://www.unicode.org/reports/tr9/
- *
- * If the input base direction is a weak direction, the direction of the
- * characters in the text will determine the final resolved direction.
- *
- * Return value: a newly allocated array of embedding levels, one item per
- *               character (not byte), that should be freed using g_free.
- *
- * Since: 1.4
- */
-guint8 *
-pango_log2vis_get_embedding_levels (const gchar    *text,
-				    int             length,
-				    PangoDirection *pbase_dir)
-{
-  FriBidiCharType fribidi_base_dir;
-  guint8 *embedding_levels_list;
-
-  switch (*pbase_dir)
-    {
-    case PANGO_DIRECTION_LTR:
-    case PANGO_DIRECTION_TTB_RTL:
-      fribidi_base_dir = FRIBIDI_TYPE_L;
-      break;
-    case PANGO_DIRECTION_RTL:
-    case PANGO_DIRECTION_TTB_LTR:
-      fribidi_base_dir = FRIBIDI_TYPE_R;
-      break;
-    case PANGO_DIRECTION_WEAK_RTL:
-      fribidi_base_dir = FRIBIDI_TYPE_WR;
-      break;
-    /*
-    case PANGO_DIRECTION_WEAK_LTR:
-    case PANGO_DIRECTION_NEUTRAL:
-    */
-    default:
-      fribidi_base_dir = FRIBIDI_TYPE_WL;
-      break;
-    }
-
-#ifdef FRIBIDI_HAVE_UTF8
-  {
-    if (length < 0)
-      length = strlen (text);
-    embedding_levels_list = fribidi_log2vis_get_embedding_levels_new_utf8 (text, length, &fribidi_base_dir);
-  }
-#else
-  {
-    gunichar *text_ucs4;
-    int n_chars;
-    text_ucs4 = g_utf8_to_ucs4_fast (text, length, &n_chars);
-    embedding_levels_list = g_new (guint8, n_chars);
-    fribidi_log2vis_get_embedding_levels ((FriBidiChar*)text_ucs4, n_chars,
-					  &fribidi_base_dir,
-					  (FriBidiLevel*)embedding_levels_list);
-    g_free (text_ucs4);
-  }
-#endif
-
-  *pbase_dir = (fribidi_base_dir == FRIBIDI_TYPE_L) ?  PANGO_DIRECTION_LTR : PANGO_DIRECTION_RTL;
-
-  return embedding_levels_list;
-}
-
-/**
- * pango_unichar_direction:
- * @ch: a Unicode character
- *
- * Determines the direction of a character; either
- * %PANGO_DIRECTION_LTR, %PANGO_DIRECTION_RTL, or
- * %PANGO_DIRECTION_NEUTRAL.
- *
- * Return value: the direction of the character, as used in the
- * Unicode bidirectional algorithm.
- */
-PangoDirection
-pango_unichar_direction (gunichar ch)
-{
-  FriBidiCharType fribidi_ch_type = fribidi_get_type (ch);
-
-  if (!FRIBIDI_IS_LETTER (fribidi_ch_type))
-    return PANGO_DIRECTION_NEUTRAL;
-  else if (FRIBIDI_IS_RTL (fribidi_ch_type))
-    return PANGO_DIRECTION_RTL;
-  else
-    return PANGO_DIRECTION_LTR;
-}
-
-/**
- * pango_get_mirror_char:
- * @ch: a Unicode character
- * @mirrored_ch: location to store the mirrored character
- *
- * If @ch has the Unicode mirrored property and there is another Unicode
- * character that typically has a glyph that is the mirror image of @ch's
- * glyph, puts that character in the address pointed to by @mirrored_ch.
- *
- * Use g_unichar_get_mirror_char() instead; the docs for that function
- * provide full details.
- *
- * Return value: %TRUE if @ch has a mirrored character and @mirrored_ch is
- * filled in, %FALSE otherwise
- **/
-gboolean
-pango_get_mirror_char (gunichar        ch,
-		       gunichar       *mirrored_ch)
-{
-  return g_unichar_get_mirror_char (ch, mirrored_ch);
-}
-
 
 static guint
 alias_hash (struct PangoAlias *alias)
@@ -1240,117 +1115,177 @@ alias_free (struct PangoAlias *alias)
   g_slice_free (struct PangoAlias, alias);
 }
 
+static void
+handle_alias_line (GString  *line_buffer,
+		   char    **errstring)
+{
+  GString *tmp_buffer1;
+  GString *tmp_buffer2;
+  const char *pos;
+  struct PangoAlias alias_key;
+  struct PangoAlias *alias;
+  gboolean append = FALSE;
+  char **new_families;
+  int n_new;
+  int i;
+
+  tmp_buffer1 = g_string_new (NULL);
+  tmp_buffer2 = g_string_new (NULL);
+
+
+  pos = line_buffer->str;
+  if (!pango_skip_space (&pos))
+    return;
+
+  if (!pango_scan_string (&pos, tmp_buffer1) ||
+      !pango_skip_space (&pos))
+    {
+      *errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
+      goto error;
+    }
+
+  if (*pos == '+')
+    {
+      append = TRUE;
+      pos++;
+    }
+
+  if (*(pos++) != '=')
+    {
+      *errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
+      goto error;
+    }
+
+  if (!pango_scan_string (&pos, tmp_buffer2))
+    {
+      *errstring = g_strdup ("Error parsing value string");
+      goto error;
+    }
+  if (pango_skip_space (&pos))
+    {
+      *errstring = g_strdup ("Junk after value string");
+      goto error;
+    }
+
+  alias_key.alias = g_ascii_strdown (tmp_buffer1->str, -1);
+
+  /* Remove any existing values */
+  alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
+
+  if (!alias)
+    {
+      alias = g_slice_new0 (struct PangoAlias);
+      alias->alias = alias_key.alias;
+      
+      g_hash_table_insert (pango_aliases_ht,
+			   alias, alias);
+    }
+  else
+    g_free (alias_key.alias);
+
+  new_families = g_strsplit (tmp_buffer2->str, ",", -1);
+
+  n_new = 0;
+  while (new_families[n_new])
+    n_new++;
+
+  if (alias->families && append)
+    {
+      alias->families = g_realloc (alias->families,
+				   sizeof (char *) *(n_new + alias->n_families));
+      for (i = 0; i < n_new; i++)
+	alias->families[alias->n_families + i] = new_families[i];
+      g_free (new_families);
+      alias->n_families += n_new;
+    }
+  else
+    {
+      for (i = 0; i < alias->n_families; i++)
+	g_free (alias->families[i]);
+      g_free (alias->families);
+      
+      alias->families = new_families;
+      alias->n_families = n_new;
+    }
+
+ error:
+  
+  g_string_free (tmp_buffer1, TRUE);
+  g_string_free (tmp_buffer2, TRUE);
+}
+
+#ifdef G_OS_WIN32
+
+static const char *builtin_aliases[] = {
+  "courier = \"courier new\"",
+  "\"segoe ui\" = \"segoe ui,arial unicode ms,browallia new,mingliu,simhei,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\"",
+  "tahoma = \"tahoma,arial unicode ms,browallia new,mingliu,simhei,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\"",
+  /* It sucks to use the same GulimChe, MS Gothic, Sylfaen, Kartika,
+   * Latha, Mangal and Raavi fonts for all three of sans, serif and
+   * mono, but it isn't like there would be much choice. For most
+   * non-Latin scripts that Windows includes any font at all for, it
+   * has ony one. One solution is to install the free DejaVu fonts
+   * that are popular on Linux. They are listed here first.
+   */
+  "sans = \"dejavu sans,tahoma,arial unicode ms,browallia new,mingliu,simhei,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\"",
+  "sans-serif = \"dejavu sans,tahoma,arial unicode ms,browallia new,mingliu,simhei,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\"",
+  "serif = \"dejavu serif,georgia,angsana new,mingliu,simsun,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\"",
+  "mono = \"dejavu sans mono,courier new,courier monothai,mingliu,simsun,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\"",
+  "monospace = \"dejavu sans mono,courier new,courier monothai,mingliu,simsun,gulimche,ms gothic,sylfaen,kartika,latha,mangal,raavi\""
+};
+
+static void
+read_builtin_aliases (void)
+{
+
+  GString *line_buffer;
+  char *errstring = NULL;
+  int line;
+
+  line_buffer = g_string_new (NULL);
+
+  for (line = 0; line < G_N_ELEMENTS (builtin_aliases) && errstring == NULL; line++)
+    {
+      g_string_assign (line_buffer, builtin_aliases[line]);
+      handle_alias_line (line_buffer, &errstring);
+    }
+
+  if (errstring)
+    {
+      g_error ("error in built-in aliases:%d: %s\n", line, errstring);
+      g_free (errstring);
+    }
+
+  g_string_free (line_buffer, TRUE);
+}
+
+#endif
+
 static void
 read_alias_file (const char *filename)
 {
   FILE *file;
 
   GString *line_buffer;
-  GString *tmp_buffer1;
-  GString *tmp_buffer2;
   char *errstring = NULL;
-  const char *pos;
   int line = 0;
-  struct PangoAlias alias_key;
-  struct PangoAlias *alias;
-  char **new_families;
-  int n_new;
-  int i;
 
   file = g_fopen (filename, "r");
   if (!file)
     return;
 
   line_buffer = g_string_new (NULL);
-  tmp_buffer1 = g_string_new (NULL);
-  tmp_buffer2 = g_string_new (NULL);
 
-  while (pango_read_line (file, line_buffer))
+  while (pango_read_line (file, line_buffer) &&
+	 errstring == NULL)
     {
-      gboolean append = FALSE;
       line++;
-
-      pos = line_buffer->str;
-      if (!pango_skip_space (&pos))
-	continue;
-
-      if (!pango_scan_word (&pos, tmp_buffer1) ||
-	  !pango_skip_space (&pos))
-	{
-	  errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
-	  goto error;
-	}
-
-      if (*pos == '+')
-	{
-	  append = TRUE;
-	  pos++;
-	}
-
-      if (*(pos++) != '=')
-	{
-	  errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
-	  goto error;
-	}
-
-      if (!pango_scan_string (&pos, tmp_buffer2))
-	{
-	  errstring = g_strdup ("Error parsing value string");
-	  goto error;
-	}
-      if (pango_skip_space (&pos))
-	{
-	  errstring = g_strdup ("Junk after value string");
-	  goto error;
-	}
-
-      alias_key.alias = g_ascii_strdown (tmp_buffer1->str, -1);
-
-      /* Remove any existing values */
-      alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
-
-      if (!alias)
-	{
-	  alias = g_slice_new0 (struct PangoAlias);
-	  alias->alias = alias_key.alias;
-
-	  g_hash_table_insert (pango_aliases_ht,
-			       alias, alias);
-	}
-      else
-	g_free (alias_key.alias);
-
-      new_families = g_strsplit (tmp_buffer2->str, ",", -1);
-
-      n_new = 0;
-      while (new_families[n_new])
-	n_new++;
-
-      if (alias->families && append)
-	{
-	  alias->families = g_realloc (alias->families,
-				       sizeof (char *) *(n_new + alias->n_families));
-	  for (i = 0; i < n_new; i++)
-	    alias->families[alias->n_families + i] = new_families[i];
-	  g_free (new_families);
-	  alias->n_families += n_new;
-	}
-      else
-	{
-	  for (i = 0; i < alias->n_families; i++)
-	    g_free (alias->families[i]);
-	  g_free (alias->families);
-
-	  alias->families = new_families;
-	  alias->n_families = n_new;
-	}
+      handle_alias_line (line_buffer, &errstring);
     }
 
-  if (ferror (file))
+  if (errstring == NULL && ferror (file))
     errstring = g_strdup (g_strerror(errno));
 
- error:
-
   if (errstring)
     {
       g_warning ("error reading alias file: %s:%d: %s\n", filename, line, errstring);
@@ -1358,8 +1293,6 @@ read_alias_file (const char *filename)
     }
 
   g_string_free (line_buffer, TRUE);
-  g_string_free (tmp_buffer1, TRUE);
-  g_string_free (tmp_buffer2, TRUE);
 
   fclose (file);
 }
@@ -1375,6 +1308,9 @@ pango_load_aliases (void)
 					    (GDestroyNotify)alias_free,
 					    NULL);
 
+#ifdef G_OS_WIN32
+  read_builtin_aliases ();
+#endif
 
   filename = g_strconcat (pango_get_sysconf_subdirectory (),
 			  G_DIR_SEPARATOR_S "pango.aliases",
@@ -1525,7 +1461,7 @@ pango_is_zero_width (gunichar ch)
 
 /**
  * pango_quantize_line_geometry:
- * @thickness: pointer to the thickness of a line, in Pango scaled units
+ * @thickness: pointer to the thickness of a line, in Pango units
  * @position: corresponding position
  *
  * Quantizes the thickness and position of a line, typically an
@@ -1533,6 +1469,10 @@ pango_is_zero_width (gunichar ch)
  * multiples of %PANGO_SCALE. The purpose of this function is to avoid
  * such lines looking blurry.
  *
+ * Care is taken to make sure @thickness is at least one pixel when this
+ * function returns, but returned @position may become zero as a result
+ * of rounding.
+ *
  * Since: 1.12
  */
 void
@@ -1593,52 +1533,53 @@ pango_units_to_double (int i)
 
 /**
  * pango_extents_to_pixels:
- * @ink_rect: ink rectangle to convert, or %NULL.
- * @logical_rect: logical rectangle to convert, or %NULL.
+ * @inclusive: rectangle to round to pixels inclusively, or %NULL.
+ * @nearest: rectangle to round to nearest pixels, or %NULL.
  *
  * Converts extents from Pango units to device units, dividing by the
  * %PANGO_SCALE factor and performing rounding.
  *
- * The ink rectangle is converted by flooring the x/y coordinates and extending
+ * The @inclusive rectangle is converted by flooring the x/y coordinates and extending
  * width/height, such that the final rectangle completely includes the original
  * rectangle.
  *
- * The logical rectangle is converted by rounding the coordinates
- * of the rectangle to the nearest device unit.
+ * The @nearest rectangle is converted by rounding the coordinates
+ * of the rectangle to the nearest device unit (pixel).
  *
- * Note that in certain situations you may want pass a logical extents
- * rectangle to this function as @ink_rect.  The rule is: if you want the
- * resulting device-space rectangle to completely contain the original
- * rectangle, pass it in as @ink_rect.
+ * The rule to which argument to use is: if you want the resulting device-space
+ * rectangle to completely contain the original rectangle, pass it in as @inclusive.
+ * If you want two touching-but-not-overlapping rectangles stay
+ * touching-but-not-overlapping after rounding to device units, pass them in
+ * as @nearest.
  *
  * Since: 1.16
  **/
 void
-pango_extents_to_pixels (PangoRectangle *ink_rect,
-			 PangoRectangle *logical_rect)
+pango_extents_to_pixels (PangoRectangle *inclusive,
+			 PangoRectangle *nearest)
 {
-  if (ink_rect)
+  if (inclusive)
     {
-      int orig_x = ink_rect->x;
-      int orig_y = ink_rect->y;
+      int orig_x = inclusive->x;
+      int orig_y = inclusive->y;
 
-      ink_rect->x = PANGO_PIXELS_FLOOR (ink_rect->x);
-      ink_rect->y = PANGO_PIXELS_FLOOR (ink_rect->y);
+      inclusive->x = PANGO_PIXELS_FLOOR (inclusive->x);
+      inclusive->y = PANGO_PIXELS_FLOOR (inclusive->y);
 
-      ink_rect->width  = PANGO_PIXELS_CEIL (orig_x + ink_rect->width ) - ink_rect->x;
-      ink_rect->height = PANGO_PIXELS_CEIL (orig_y + ink_rect->height) - ink_rect->y;
+      inclusive->width  = PANGO_PIXELS_CEIL (orig_x + inclusive->width ) - inclusive->x;
+      inclusive->height = PANGO_PIXELS_CEIL (orig_y + inclusive->height) - inclusive->y;
     }
 
-  if (logical_rect)
+  if (nearest)
     {
-      int orig_x = logical_rect->x;
-      int orig_y = logical_rect->y;
+      int orig_x = nearest->x;
+      int orig_y = nearest->y;
 
-      logical_rect->x = PANGO_PIXELS (logical_rect->x);
-      logical_rect->y = PANGO_PIXELS (logical_rect->y);
+      nearest->x = PANGO_PIXELS (nearest->x);
+      nearest->y = PANGO_PIXELS (nearest->y);
 
-      logical_rect->width  = PANGO_PIXELS (orig_x + logical_rect->width ) - logical_rect->x;
-      logical_rect->height = PANGO_PIXELS (orig_y + logical_rect->height) - logical_rect->y;
+      nearest->width  = PANGO_PIXELS (orig_x + nearest->width ) - nearest->x;
+      nearest->height = PANGO_PIXELS (orig_y + nearest->height) - nearest->y;
     }
 }
 
