@@ -33,8 +33,6 @@
 #include "pangofc-fontmap.h"
 #include "pangofc-private.h"
 
-PangoFT2WarningHistory _pango_ft2_warning_history = { FALSE };
-
 /* for compatibility with older freetype versions */
 #ifndef FT_LOAD_TARGET_MONO
 #define FT_LOAD_TARGET_MONO  FT_LOAD_MONOCHROME
@@ -51,15 +49,16 @@ struct _PangoFT2FontClass
   PangoFcFontClass parent_class;
 };
 
-static void pango_ft2_font_finalize   (GObject         *object);
+static void     pango_ft2_font_finalize          (GObject        *object);
 
-static void                   pango_ft2_font_get_glyph_extents (PangoFont      *font,
-								PangoGlyph      glyph,
-								PangoRectangle *ink_rect,
-								PangoRectangle *logical_rect);
+static void     pango_ft2_font_get_glyph_extents (PangoFont      *font,
+                                                  PangoGlyph      glyph,
+                                                  PangoRectangle *ink_rect,
+                                                  PangoRectangle *logical_rect);
+
+static FT_Face  pango_ft2_font_real_lock_face    (PangoFcFont    *font);
+static void     pango_ft2_font_real_unlock_face  (PangoFcFont    *font);
 
-static FT_Face    pango_ft2_font_real_lock_face         (PangoFcFont      *font);
-static void       pango_ft2_font_real_unlock_face       (PangoFcFont      *font);
 
 PangoFT2Font *
 _pango_ft2_font_new (PangoFT2FontMap *ft2fontmap,
@@ -100,7 +99,7 @@ load_fallback_face (PangoFT2Font *ft2font,
 			 FC_PIXEL_SIZE, FcTypeDouble, (double)ft2font->size / PANGO_SCALE,
 			 NULL);
 
-  _pango_ft2_font_map_default_substitute (fcfont->fontmap, sans);
+  _pango_ft2_font_map_default_substitute ((PangoFcFontMap *)fcfont->fontmap, sans);
 
   matched = FcFontMatch (NULL, sans, &result);
 
@@ -118,8 +117,7 @@ load_fallback_face (PangoFT2Font *ft2font,
     {
     bail1:
       name = pango_font_description_to_string (fcfont->description);
-      g_warning ("Unable to open font file %s for font %s, exiting\n", filename2, name);
-      exit (1);
+      g_error ("Unable to open font file %s for font %s, exiting\n", filename2, name);
     }
   else
     {
@@ -162,7 +160,8 @@ set_transform (PangoFT2Font *ft2font)
  * face from pango_fc_font_lock_face() you must call
  * pango_fc_font_unlock_face().
  *
- * Return value: a pointer to a <type>FT_Face</type> structure, with the size set correctly
+ * Return value: a pointer to a <type>FT_Face</type> structure, with the size set correctly,
+ *               or %NULL if @font is %NULL.
  **/
 FT_Face
 pango_ft2_font_get_face (PangoFont *font)
@@ -173,17 +172,11 @@ pango_ft2_font_get_face (PangoFont *font)
   FcPattern *pattern;
   FcChar8 *filename;
   FcBool antialias, hinting, autohint;
+  int hintstyle;
   int id;
 
-  if (G_UNLIKELY (!PANGO_FT2_IS_FONT (font)))
-    {
-      if (!_pango_ft2_warning_history.get_face)
-	{
-	  _pango_ft2_warning_history.get_face = TRUE;
-	  g_warning ("pango_ft2_font_get_face called with bad font, expect ugly output");
-	}
-      return NULL;
-    }
+  if (G_UNLIKELY (!font))
+    return NULL;
 
   pattern = fcfont->font_pattern;
 
@@ -206,8 +199,26 @@ pango_ft2_font_get_face (PangoFont *font)
 			    FC_HINTING, 0, &hinting) != FcResultMatch)
 	hinting = FcTrue;
 
+#ifdef FC_HINT_STYLE
+      if (FcPatternGetInteger (pattern, FC_HINT_STYLE, 0, &hintstyle) != FcResultMatch)
+	hintstyle = FC_HINT_FULL;
+
+      if (!hinting || hintstyle == FC_HINT_NONE)
+          ft2font->load_flags |= FT_LOAD_NO_HINTING;
+      
+      switch (hintstyle) {
+      case FC_HINT_SLIGHT:
+      case FC_HINT_MEDIUM:
+	ft2font->load_flags |= FT_LOAD_TARGET_LIGHT;
+	break;
+      default:
+	ft2font->load_flags |= FT_LOAD_TARGET_NORMAL;
+	break;
+      }
+#else
       if (!hinting)
-	ft2font->load_flags |= FT_LOAD_NO_HINTING;
+          ft2font->load_flags |= FT_LOAD_NO_HINTING;
+#endif
 
       /* force autohinting if requested */
       if (FcPatternGetBool (pattern,
@@ -318,40 +329,35 @@ pango_ft2_font_get_glyph_extents (PangoFont      *font,
 
   if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
     {
-      glyph = pango_ft2_get_unknown_glyph (font);
-      if (glyph == PANGO_GLYPH_EMPTY)
+      PangoFontMetrics *metrics = pango_font_get_metrics (font, NULL);
+
+      if (metrics)
 	{
-	  /* No unknown glyph found for the font, draw a box */
-	  PangoFontMetrics *metrics = pango_font_get_metrics (font, NULL);
-
-	  if (metrics)
+	  if (ink_rect)
 	    {
-	      if (ink_rect)
-		{
-		  ink_rect->x = PANGO_SCALE;
-		  ink_rect->width = metrics->approximate_char_width - 2 * PANGO_SCALE;
-		  ink_rect->y = - (metrics->ascent - PANGO_SCALE);
-		  ink_rect->height = metrics->ascent + metrics->descent - 2 * PANGO_SCALE;
-		}
-	      if (logical_rect)
-		{
-		  logical_rect->x = 0;
-		  logical_rect->width = metrics->approximate_char_width;
-		  logical_rect->y = -metrics->ascent;
-		  logical_rect->height = metrics->ascent + metrics->descent;
-		}
-
-	      pango_font_metrics_unref (metrics);
+	      ink_rect->x = PANGO_SCALE;
+	      ink_rect->width = metrics->approximate_char_width - 2 * PANGO_SCALE;
+	      ink_rect->y = - (metrics->ascent - PANGO_SCALE);
+	      ink_rect->height = metrics->ascent + metrics->descent - 2 * PANGO_SCALE;
 	    }
-	  else
+	  if (logical_rect)
 	    {
-	      if (ink_rect)
-		ink_rect->x = ink_rect->y = ink_rect->height = ink_rect->width = 0;
-	      if (logical_rect)
-		logical_rect->x = logical_rect->y = logical_rect->height = logical_rect->width = 0;
+	      logical_rect->x = 0;
+	      logical_rect->width = metrics->approximate_char_width;
+	      logical_rect->y = -metrics->ascent;
+	      logical_rect->height = metrics->ascent + metrics->descent;
 	    }
-	  return;
+
+	  pango_font_metrics_unref (metrics);
 	}
+      else
+	{
+	  if (ink_rect)
+	    ink_rect->x = ink_rect->y = ink_rect->height = ink_rect->width = 0;
+	  if (logical_rect)
+	    logical_rect->x = logical_rect->y = logical_rect->height = logical_rect->width = 0;
+	}
+      return;
     }
 
   info = pango_ft2_font_get_glyph_info (font, glyph, TRUE);
@@ -457,7 +463,8 @@ pango_ft2_font_finalize (GObject *object)
  * @language: a language tag.
  * @returns: a #PangoCoverage.
  *
- * Gets the #PangoCoverage for a #PangoFT2Font. Use pango_font_get_coverage() instead.
+ * Gets the #PangoCoverage for a #PangoFT2Font. Use
+ * pango_font_get_coverage() instead.
  **/
 PangoCoverage *
 pango_ft2_font_get_coverage (PangoFont     *font,
