@@ -178,7 +178,7 @@ pango_x_font_map_get_type (void)
 {
   static GType object_type = 0;
 
-  if (!object_type)
+  if (G_UNLIKELY (!object_type))
     {
       const GTypeInfo object_info =
       {
@@ -374,7 +374,12 @@ pango_x_font_map_finalize (GObject *object)
 
   pango_x_font_cache_free (xfontmap->font_cache);
 
-  /* FIXME: Lots more here */
+  /* FIXME: None of these hashtables free their key/values
+  g_hash_table_destroy (xfontmap->families);
+  g_hash_table_destroy (xfontmap->size_infos);
+  g_hash_table_destroy (xfontmap->to_atom_cache);
+  g_hash_table_destroy (xfontmap->from_atom_cache);
+  */
 
   fontmaps = g_list_remove (fontmaps, xfontmap);
 
@@ -640,7 +645,7 @@ pango_x_get_coverage_win (PangoXFontMap *xfontmap)
 	    XCreateWindow (persistant_display,
 			   DefaultRootWindow (persistant_display),
 			   -100, -100, 10, 10, 0, 0,
-			   InputOnly, CopyFromParent,
+			   InputOnly, (Visual *)CopyFromParent,
 			   CWOverrideRedirect, &attr);
 
 	  XChangeProperty (persistant_display,
@@ -1100,7 +1105,7 @@ pango_x_insert_font (PangoXFontMap *xfontmap,
   size_info = g_hash_table_lookup (xfontmap->size_infos, identifier);
   if (!size_info)
     {
-      size_info = g_new (PangoXSizeInfo, 1);
+      size_info = g_slice_new (PangoXSizeInfo);
       size_info->identifier = identifier;
       size_info->xlfds = NULL;
 
@@ -1490,7 +1495,7 @@ pango_x_face_get_type (void)
 {
   static GType object_type = 0;
 
-  if (!object_type)
+  if (G_UNLIKELY (!object_type))
     {
       const GTypeInfo object_info =
       {
@@ -1649,7 +1654,7 @@ pango_x_family_get_type (void)
 {
   static GType object_type = 0;
 
-  if (!object_type)
+  if (G_UNLIKELY (!object_type))
     {
       const GTypeInfo object_info =
       {
